''The program estimates  AR(1) models with asymptotically subsiding trends 
'        as well as ordinary (linear) AR(1) without constant and 
'        with constant and tests them for unit root through the Phillips-Perron and 
'        ADF tests for pairs of every time series y[r] in the specified group with a
'        specified series (benchmark).
'        That is, seres y*[r]=y[r]-y* are analyzed, y* being the benchmark serires.
'        The trends are:
'            log-exponential: ln(1 + gamma*exp(delta*t)), 
'            exponential:        gamma*exp(delta*t), 
'            and fractional:    gamma/(1 + delta*t).

'         Version: 2020.07

'         Author: Konstantin Gluschenko
'                       glu@nsu.ru
'                       http://nsu-ef.ru/teachers/Gluschenko_KP/
 
'The program stores results obtained in table ESTIMATES_benchmark name.

'ARGUMENTS of the program
'          %0: name of group with series to be analyzed
'          %1: name of the benchmark series (its may be within or beyond the group)
'          Examples: states zzNational
'                               Mexican_States  AGUASCALIENTES 

'Load a workfile with a group of series to be analyzed before loadng the program.

' Set "Maximum errors before halting" in the Run dialog  to a number 10 or more!

'====================== SUBROUTINES ======================

'************ Read distribution of tau-statistic from Excel file  ****************
Subroutine Get_tau(string %Sheet, vector tau, scalar T, scalar Sample_size_used)
RowVector(255) Sample_size=NA

!errors=@errorcount
p.read(t=xls,rect,s={%Sheet},A7)  {%CVfile}
     if !errors<>@errorcount then
          StatusLine Error while reading {%CVfile} or it does not contain a sheet named {%Sheet}
          STOP
     endif
!errors=@errorcount
Sample_size.read(t=xls,rect,s={%Sheet},B6)  {%CVfile} 'Read line of sample sizes
     for !c=255 to 1 step -1
          if Sample_size(!c)<=T or !c=1 then
               exitloop
          endif
     next
     if Sample_size(!c)=NA then
          StatusLine Error(s) in sample sizes specified in {%CVfile}, sheet {%Sheet}
          STOP
     endif
Sample_size_used=Sample_size(!c)

'Compute the letter symbol of column in %CVfile 
!c=!c+1 
%ABC="ABCDEFGHIJKLMNOPQRSTUVWXYZ"
%Column=""
     !f=@floor(!c/26-0.03)
          if !f>0 then
               %Column=%Column+@mid(%ABC,!f,1)
          endif
!s=!c-!f*26
%Column=%Column+@mid(%ABC,!s,1)
%Cell=%Column+"7"

'Read tau-statistic
tau.read(t=xls,rect,s={%Sheet},{%Cell})  {%CVfile}
     for !i=1 to !N_of_p_values
          if p(!i)=NA or tau(!i)=NA then
               !i=!i+6
               StatusLine Error in the tau-statisic data. See cell A!i or {%Column}!i in {%CVfile}, sheet {%Sheet}.
               STOP
          endif
     next
Delete Sample_size
EndSub 'Get_tau

'******************* Generate initial values of coefficients *********************
Subroutine Best_Fit(series y, string %trend, string %Equation, string %Lags, vector coef)
!minSSR=1e20

' 1. Initial approximation of coefs is 0
c=0
Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)

' 2. Initial approximation of coefs is c for ARo
c=0
Equation ARo.ls D(y)=c(1)*y(-1)
Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)

' 3. Initial approximation of coefs is c for ARc
c=0
Equation ARc.ls D(y)=c(1)*y(-1)+c(4)
c(2)=-c(4)/c(1)
     if c(2)<20 then
               if %trend="log" then
                    c(2)=exp(-c(2))-1
               endif
           Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
     endif        

' 4. Linear approximation
c=0
Equation ARt.ls D(y)=c(1)*y(-1)+c(4)+c(5)*@trend
c(2)=-(c(1)*c(4)+(c(1)+1)*c(5))/(c(1)*c(1))
c(3)=-c(5)/(c(1)*c(2))
     if %trend="fract" and c(3)*(!T_-1)<1 then
          c(3)=-c(3)
          Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
     endif
     if %trend<>"fract" and c(3)*(!T_-1)<20 then
               if %trend<>"log" or (%trend="log" and c(2)>-1 and c(2)*exp(c(3)*(!T_-1))>-1) then 
                    Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
               endif
     endif

' 5. Semi-linear approximation: exp(x) -> 1 + x
     if %trend="log" then
          c=0
          !errors=@errorcount
          Equation ARt1.ls(M=1000,C=1e-05) D(y)=c(1)*y(-1)+log(1+c(2)*(1+c(3)*@trend))-(c(1)+1)*log(1+c(2)*(1+c(3)*(@trend-1)))
               if !errors=@errorcount and c(2)>-1 and c(3)*(!T_-1)<20 then
                         if c(2)*exp(c(3)*(!T_-1))>-1 then    
                              Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
                         endif
               endif
      endif

' 6. Semi-linear approximation: ln(1+ x) -> x, simplified exp trend
     if %trend<>"fract" then
          c=0
          !errors=@errorcount
          Equation ARt2.ls(M=1000,C=1e-05) D(y)=c(1)*y(-1)+c(2)*exp(c(3)*@trend)*(1-(c(1)+1)*exp(-c(3)))
          c(2)=c(2)/(1-(c(1)+1)*exp(-c(3)))
               if !errors=@errorcount and (%trend="exp" and c(2)<20) or (%trend="log" and c(2)>-1 and c(2)*exp(c(3)*(!T_-1))>-1 and c(3)*(!T_-1)<20) then          
                    Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
               endif
     endif

' 7. Initial approximation is computed thru detrending
c=0
!errors=@errorcount
     if %trend="log" then
          Equation Detrend_log.ls(M=1000,C=1e-05) y=log(1+c(2)*exp(c(3)*@trend))
     endif
     if %trend="exp" then
          Equation Detrend_exp.ls(M=1000,C=1e-05) y=c(2)*exp(c(3)*@trend)
     endif
     if %trend="fract" then
          Equation Detrend_fract.ls(M=1000,C=1e-05) y=c(2)/(1+c(3)*@trend)
     endif
     if !errors=@errorcount then
          series eps=resid
          Equation ARo_eps.ls D(eps)=c(1)*eps(-1)
          Call Estimate(y,%Equation, %Lags, %trend, coef, !minSSR)
     endif
EndSub 'Best_Fit

'********** Estimate model with some initial values of coefficients ************
Subroutine Estimate(series y, string %Equation, string %Lags, string %trend, vector coef, scalar minSSR)

!errors=@errorcount
Equation NL_.ls(M=1000,C=1e-07) {%Equation}{%Lags}

     'Select and store the best estimate
     if !errors=@errorcount then
          Freeze(Eq) NL_.results
          %conv=@left(Eq(6,1),24)
          !fract_err=(%trend="fract" and c(3)*(!T_-1)<=-1)  
               if @stderrs(1)<>NA AND minSSR>@SSR and %conv<>"Convergence not achieved" and !fract_err=0 then
                    minSSR=@SSR
                         for !i=1 to 5+!lagmax
                              coef(!i)=c(!i)     'lambda
                         next
                endif
          Delete Eq
     endif
EndSub 'Estimate

'***************** Perform ADF and Phillips-Perron tests *******************
Subroutine Unit_Root_Tests(series y, series y_detrended, string %trend, string %Equation, scalar SSR_original, scalar se_lambda, scalar tau_lambda, scalar se_resid, vector tau_statistic, scalar lag_length, scalar ADF_t, scalar ADF_p, scalar PP_t, scalar PP_p)

URoot(ADF, none, info=MSIC, save=ADF) y_detrended
lag_length=ADF(2,1)  'optimal number of lags
%Lags=""
     if !lag_length<>NA then
               for !lag=1 to !lag_length
                    %Lags=%Lags+"+c("+@str(!lag+5)+")*D(y("+@str(-!lag)+"))"
               next '!lag
          Call  Best_Fit(y,  %trend, %Equation,%Lags, coefNL)
               for !i=1 to 5+!lag_length
                    c(!i)=coefNL(!i) 
               next '!i
          Equation ADFlag.ls(M=1000,C=1e-07) {%Equation}{%Lags}
               if @stderrs(1)<>NA then  
                    ADF_t=@tstats(1)     'tau ADF
                    '------------------- Compute the Phillips-Perron statistic
                    !betas=1
                         for !i=1 to !lag_length
                              !betas=!betas-c(!i+5)
                         next
                    !g0=SSR_original/(!T_-1)
                    !f0=(SSR_original/(!T_-1))/(!betas*!betas) 
                    PP_t=tau_lambda*sqr(!g0/!f0)-(!T_-1)*(!f0-!g0)*se_lambda/(2*sqr(!f0)*se_resid)

                    '-------------------- Find p-values 
                    PP_p=1
                    ADF_p=1
                         for !i=1 to 1001
                                   if PP_t<=tau_statistic(!i) and PP_p=1 then
                                        PP_p=p(!i) 
                                    endif
                                    if ADF_t<=tau_statistic(!i) and ADF_p=1 then
                                         ADF_p=p(!i) 
                                    endif
                                    if PP_p<1 and ADF_p<1 then
                                         exitloop 
                                    endif
                         next '!i
                    delete ADF
                    delete ADFlag
               else
                    ADF_t=NA
                    ADF_p=NA
                    PP_t=NA
                    PP_p=NA
               endif '@tstats(1)<>NA 
     endif '!lag_length<>NA
EndSub 'Unit_Root_Tests

'====================== PROGRAM BODY ====================

'********************** PREPARATORY PROCEDURES ********************
StatusLine
%Time="Started: "+@datestr(@now, "dd/MM/yyyy HH:mi")+"; elapsed time "
tic        'Reset timer

'Check program arguments
%0=@upper(%0)                'Convert to capital letters
     if %0="" or @isobject(%0)=0 then
          StatusLine There is no group %0 in your workfile!
          STOP
     endif
%1=@upper(%1)               'Convert to capital letters
     if %1="" or @isobject(%1)=0 then
          StatusLine There is no benchmark series %1 in your workfile!
          STOP
     endif 
!N_={%0}.@count               'Number of locations
!T_={%0}.@maxobs           'Sample size
!lagmax=@floor(12*((!T_-1)/100)^0.25)    'maximum number of lags

vector(5+!lagmax) coefNL=NA
Series Yrs
Series Y_detrended
!N_of_p_values=1001
!Significance_level=0.1
Table(3,1) Tau_Status

'****************** ESTIMATING AND TESTING: LOG-EXPONENTIAL TREND 
'--------------- Read tau-statistic
%CVfile="TauStatistics-NonlinearTrends.xls"
!sample_size_used=NA
Vector(!N_of_p_values) p=NA  
Vector(!N_of_p_values) tau_log=NA
Call Get_tau("Log-exp", tau_log, !T_, !sample_size_used)
Tau_Status(1,1)="Log-exponential trend: tau-statistic for T="+@str(!T_)
     if !sample_size_used<>!T_ then
          Ln_Status(1,1)="Log-exponential trend: No tau-statistic for T="+@str(!T_)+" in "+%CVfile+". This for T="+@str(!sample_size_used)+" is used instead."
     endif

'--------------- Estimate
Matrix(!N_,17) Ln_Est=NA
%Eq_log_trend="D(y)=c(1)*y(-1)+log(1+c(2)*exp(c(3)*@trend))-(c(1)+1)*log(1+c(2)*exp(c(3)*(@trend -1)))"

     for !r=1 to !N_
          StatusLine log trend: r=!r
          Series Yrs={%0}(!r)-{%1}
               if {%0}.@seriesname(!r)<>%1 then
                    coefNL=NA
                    !PP_t=NA
                    !PPp=NA
                    !ADF_t=NA
                    !ADFp=NA
                    !lag_length=NA
                    Call Best_Fit(Yrs, "log", %Eq_log_trend,  "", coefNL)
                    c(1)=coefNL(1)
                    c(2)=coefNL(2)
                    c(3)=coefNL(3)
                         if coefNL(1)<>NA then
                              Equation Log_exp_trend.ls(M=1000,C=1e-07) D(Yrs)=c(1)*Yrs(-1)+log(1+c(2)*exp(c(3)*@trend))-(c(1)+1)*log(1+c(2)*exp(c(3)*(@trend -1)))
                         endif
                         if coefNL(1)<>NA and @stderrs(1)<>NA then
                              Ln_Est(!r,1)=c(1)                     'lambda 
                              Ln_Est(!r,2)=@stderrs(1)
                              Ln_Est(!r,3)=@tstats(1)
                              Ln_Est(!r,10)=c(2)                   'gamma
                              Ln_Est(!r,11)=@stderrs(2)  
                              Ln_Est(!r,12)=@tdist(@tstats(2), @regobs-@ncoef)
                              Ln_Est(!r,13)=c(3)                     'delta 
                              Ln_Est(!r,14)=@stderrs(3)
                              Ln_Est(!r,15)=@tdist(@tstats(3), @regobs-@ncoef)
                              Ln_Est(!r,16)=@rbar2
                              Ln_Est(!r,17)=@SSR                              
'--------------- Perform unit root test
                              Y_detrended=Yrs - log(1+c(2)*exp(c(3)*@trend))
                              call Unit_Root_Tests(Yrs, Y_detrended, "log", %Eq_log_trend, Log_exp_trend.@SSR, Log_exp_trend.@stderrs(1), Log_exp_trend.@tstats(1), Log_exp_trend.@se, tau_log, !lag_length, !ADF_t, !ADFp, !PP_t, !PPp)
                              Ln_Est(!r,4)= !PP_t
                              Ln_Est(!r,6)=!lag_length
                              Ln_Est(!r,7)= !ADF_t
                              Ln_Est(!r,9)=!lag_length
                              Ln_Est(!r,5)= !PPp
                              Ln_Est(!r,8)=!ADFp
                         endif 'coefNL(1)<>NA and @stderrs(1)<>NA                         
               endif '{%0}.@seriesname(!r)<>%1
     next 'r 

'****************** ESTIMATING AND TESTING: EXPONENTIAL TREND 
'--------------- Read tau-statistic
%CVfile="TauStatistics-NonlinearTrends.xls"
!sample_size_used=NA
Vector(!N_of_p_values) p=NA  
Vector(!N_of_p_values) tau_exp=NA
Call Get_tau("Exp", tau_exp, !T_, !sample_size_used)
Tau_Status(2,1)="Exponential trend: tau-statistic for T="+@str(!T_)
     if !sample_size_used<>!T_ then
          Ln_Status(2,1)="Exponential trend: No tau-statistic for T="+@str(!T_)+" in "+%CVfile+". This for T="+@str(!sample_size_used)+" is used instead."
     endif

'--------------- Estimate
Matrix(!N_,17) Exp_Est=NA
%Eq_exp_trend="D(y)=c(1)*y(-1)+c(2)*exp(c(3)*@trend)-(c(1)+1)*c(2)*exp(c(3)*(@trend -1))"

     for !r=1 to !N_
          StatusLine exp trend: r=!r
          Series Yrs={%0}(!r)-{%1}
               if {%0}.@seriesname(!r)<>%1 then
                    coefNL=NA
                    !PP_t=NA
                    !PPp=NA
                    !ADF_t=NA
                    !ADFp=NA
                    !lag_length=NA
                    Call Best_Fit(Yrs, "exp", %Eq_exp_trend,  "", coefNL)
                    c(1)=coefNL(1)
                    c(2)=coefNL(2)
                    c(3)=coefNL(3)
                         if coefNL(1)<>NA then
                              Equation Exp_trend.ls(M=1000,C=1e-07) D(Yrs)=c(1)*Yrs(-1)+c(2)*exp(c(3)*@trend)-(c(1)+1)*c(2)*exp(c(3)*(@trend -1))
                         endif
                         if coefNL(1)<>NA and @stderrs(1)<>NA then
                              Exp_Est(!r,1)=c(1)                     'lambda 
                              Exp_Est(!r,2)=@stderrs(1)
                              Exp_Est(!r,3)=@tstats(1)
                              Exp_Est(!r,10)=c(2)                   'gamma
                              Exp_Est(!r,11)=@stderrs(2)  
                              Exp_Est(!r,12)=@tdist(@tstats(2), @regobs-@ncoef)
                              Exp_Est(!r,13)=c(3)                     'delta 
                              Exp_Est(!r,14)=@stderrs(3)
                              Exp_Est(!r,15)=@tdist(@tstats(3), @regobs-@ncoef)
                              Exp_Est(!r,16)=@rbar2
                              Exp_Est(!r,17)=@SSR                              
'--------------- Perform unit root test
                              Y_detrended=Yrs - c(2)*exp(c(3)*@trend)
                              call Unit_Root_Tests(Yrs, Y_detrended, "exp", %Eq_exp_trend, Exp_trend.@SSR, Exp_trend.@stderrs(1), Exp_trend.@tstats(1), Exp_trend.@se, tau_exp, !lag_length, !ADF_t, !ADFp, !PP_t, !PPp)
                              Exp_Est(!r,4)= !PP_t
                              Exp_Est(!r,6)=!lag_length
                              Exp_Est(!r,7)= !ADF_t
                              Exp_Est(!r,9)=!lag_length
                              Exp_Est(!r,5)= !PPp
                              Exp_Est(!r,8)=!ADFp
                         endif 'coefNL(1)<>NA and @stderrs(1)<>NA                         
               endif '{%0}.@seriesname(!r)<>%1
     next 'r 

'****************** ESTIMATING AND TESTING: FRACTIONAL TREND 
'--------------- Read tau-statistic
%CVfile="TauStatistics-NonlinearTrends.xls"
!sample_size_used=NA
Vector(!N_of_p_values) p=NA
Vector(!N_of_p_values) tau_fract=NA
Call Get_tau("Fract", tau_fract, !T_, !sample_size_used)
Tau_Status(3,1)="Fractional trend: tau-statistic for T="+@str(!T_)
     if !sample_size_used<>!T_ then
          Ln_Status(3,1)="Fractional trend: No tau-statistic for T="+@str(!T_)+" in "+%CVfile+". This for T="+@str(!sample_size_used)+" is used instead."
     endif

'--------------- Estimate
Matrix(!N_,17) Fract_Est=NA
%Eq_fract_trend="D(y)=c(1)*y(-1)+c(2)/(1+c(3)*@trend)-(c(1)+1)*c(2)/(1+c(3)*(@trend-1))"

     for !r=1 to !N_
          StatusLine fract trend: r=!r
          Series Yrs={%0}(!r)-{%1}
               if {%0}.@seriesname(!r)<>%1 then
                    coefNL=NA
                    !PP_t=NA
                    !PPp=NA
                    !ADF_t=NA
                    !ADFp=NA
                    !lag_length=NA
                    Call Best_Fit(Yrs, "fract", %Eq_fract_trend,  "", coefNL)
                    c(1)=coefNL(1)
                    c(2)=coefNL(2)
                    c(3)=coefNL(3)
                         if coefNL(1)<>NA then
                              Equation Fract_trend.ls(M=1000,C=1e-07) D(Yrs)=c(1)*Yrs(-1)+c(2)/(1+c(3)*@trend)-(c(1)+1)*c(2)/(1+c(3)*(@trend-1))
                         endif
                         if coefNL(1)<>NA and @stderrs(1)<>NA then
                              Fract_Est(!r,1)=c(1)                     'lambda 
                              Fract_Est(!r,2)=@stderrs(1)
                              Fract_Est(!r,3)=@tstats(1)
                              Fract_Est(!r,10)=c(2)                   'gamma
                              Fract_Est(!r,11)=@stderrs(2)  
                              Fract_Est(!r,12)=@tdist(@tstats(2), @regobs-@ncoef)
                              Fract_Est(!r,13)=c(3)                     'delta 
                              Fract_Est(!r,14)=@stderrs(3)
                              Fract_Est(!r,15)=@tdist(@tstats(3), @regobs-@ncoef)
                              Fract_Est(!r,16)=@rbar2
                              Fract_Est(!r,17)=@SSR                              
'--------------- Perform unit root test
                              Y_detrended=Yrs - c(2)/(1+c(3)*@trend)
                              call Unit_Root_Tests(Yrs, Y_detrended, "fract", %Eq_fract_trend, Fract_trend.@SSR, Fract_trend.@stderrs(1), Fract_trend.@tstats(1), Fract_trend.@se, tau_fract, !lag_length, !ADF_t, !ADFp, !PP_t, !PPp)
                              Fract_Est(!r,4)= !PP_t
                              Fract_Est(!r,6)=!lag_length
                              Fract_Est(!r,7)= !ADF_t
                              Fract_Est(!r,9)=!lag_length
                              Fract_Est(!r,5)= !PPp
                              Fract_Est(!r,8)=!ADFp
                         endif 'coefNL(1)<>NA and @stderrs(1)<>NA                         
               endif '{%0}.@seriesname(!r)<>%1
     next 'r 

'******************ANALIZING AR(1) AND SUMMARIIZING *********************

%AR="ARdt"
' ATTENTION! This is the value of parameter 'hac' in commands URoot(PP,...) below.
'Its correct value forExp this program would be %AR="AR".
'There is a mistake in EViews7: it executes the command with 'hac=ARdt' as if is 'hac=AR'. 
'If this mistake is corrected in a later version of EViews, change this operator to   %AR="AR" 

     if @IsObject("Estimates")=1 then
          Delete Estimates
     endif
Table(!N_*5+6,22) Estimates
Estimates.title Benchmark: {%1}
Estimates.setjust(@all) right
Estimates(1,1)="No."
   Estimates.setjust(A) left
   Estimates.setwidth(A) 4
   Estimates.setformat(A) f.0
Estimates(1,2)="Location name"
   Estimates.setwidth(B) 12
   Estimates.setjust(B) left
Estimates(1,3)="Specification"
   Estimates.setwidth(C) 6
Estimates(1,4)="lambda"
Estimates(1,5)="se"
Estimates(1,6)="t"
Estimates(1,7)="PP-t"
Estimates(1,8)="p-value"
   Estimates.setformat(H) f.3
   Estimates.setwidth(H) 8
Estimates(1,9)="Bandwidth"
   Estimates.setwidth(I) 5
   Estimates.setformat(I) f.0
Estimates(1,10)="ADF-t"
Estimates(1,11)="p-value"
   Estimates.setformat(K) f.3
   Estimates.setwidth(K) 8
Estimates(1,12)="lags"
   Estimates.setformat(L) f.0
   Estimates.setwidth(L) 5
Estimates(1,13)="gamma"
Estimates(1,14)="se"
Estimates(1,15)="p-value"
Estimates(1,16)="delta"
Estimates(1,17)="se"
Estimates(1,18)="p-value"
Estimates(1,19)="AdjR2"
Estimates(1,20)="SSR"
Estimates(1,21)="Disparity, %"
Estimates(1,22)="Halflife"

     for !r=1 to !N_
          StatusLine Summarizing r=!r
          Estimates((!r-1)*5+2,1)=!r
          Estimates((!r-1)*5+2,2)={%0}.@seriesname(!r)
          if  {%0}.@seriesname(!r)=%1 then
               Estimates((!r-1)*5+2,3)="Benchmark location"
          else
          'Significance of trend equations (check of p-values)
          !SSR_Ln=1e10
               if Ln_Est(!r,5)<=!Significance_level and Ln_Est(!r,8)<=!Significance_level and Ln_Est(!r,12)<!Significance_level and Ln_Est(!r,15)<=0.1 then
                   !SSR_Ln=Ln_Est(!r,17)
               endif
          !SSR_Exp=1e10
               if Exp_Est(!r,5)<=!Significance_level and Exp_Est(!r,8)<=!Significance_level and Exp_Est(!r,12)<=!Significance_level and Exp_Est(!r,15)<=0.1 then
                   !SSR_Exp=Exp_Est(!r,17)
               endif
          !SSR_Fract=1e10
               if Fract_Est(!r,5)<=!Significance_level and Fract_Est(!r,8)<=!Significance_level and Fract_Est(!r,12)<=!Significance_level and Fract_Est(!r,15)<=0.1 then
                   !SSR_Fract=Fract_Est(!r,17)
               endif
'----------------------- log-exponential trend
          !row=(!r-1)*5+2
          Estimates(!row,3)="log"
               if !SSR_Ln<1e10 and !SSR_Ln<=!SSR_Exp and !SSR_Ln<=!SSR_Fract then
                    Estimates.setfont(!row,3) +b
               endif
               if Ln_Est(!r,5)>!Significance_level  OR Ln_Est(!r,5)=NA  then
                    Estimates.setfillcolor(!row,8) yellow
               endif
               if Ln_Est(!r,8)>!Significance_level  OR Ln_Est(!r,8)=NA   then
                    Estimates.setfillcolor(!row,11) yellow
               endif
               if Ln_Est(!r,12)>!Significance_level  OR Ln_Est(!r,12)=NA   then
                    Estimates.setfillcolor(!row,15) yellow
               endif
               if Ln_Est(!r,15)>!Significance_level  OR Ln_Est(!r,15)=NA   then
                    Estimates.setfillcolor(!row,18) yellow
               endif
          !gamma=Ln_Est(!r,10)
          !delta=Ln_Est(!r,13)
               if !gamma=NA then
                    Estimates(!row,4)="Estimation impossible"
               else 
                         for !k=1 to 17
                              Estimates(!row,3+!k)=Ln_Est(!r,!k)
                         next
                 Estimates(!row,21)=!gamma*100   'Disparty, %    
                         if !delta<>0 then
                                   if !delta<0 then
                                        !halflife=log(0.5)/!delta
                                   else
                                       !halflife=log(2)/!delta
                                       Estimates.setfillcolor(!row,16) red
                                       Estimates.setfillcolor(!row,22) red
                                   endif
                              Estimates(!row,22)=!halflife
                         endif '!delta<>0
               endif '!gamma=NA

'----------------------- exponential trend
          !row=(!r-1)*5+3
          Estimates((!r-1)*5+3,3)="exp"
               if !SSR_Exp<1e10 and !SSR_Exp<=!SSR_Ln and !SSR_Exp<=!SSR_Fract then
                    Estimates.setfont(!row,3) +b            
               endif
               if Exp_Est(!r,5)>!Significance_level OR Exp_Est(!r,5)=NA  then
                    Estimates.setfillcolor(!row,8) yellow
               endif
               if Exp_Est(!r,8)>!Significance_level OR Exp_Est(!r,8)=NA   then
                    Estimates.setfillcolor(!row,11) yellow
               endif
               if Exp_Est(!r,12)>!Significance_level OR Exp_Est(!r,12)=NA   then
                    Estimates.setfillcolor(!row,15) yellow
               endif
               if Exp_Est(!r,15)>!Significance_level  OR Exp_Est(!r,15)=NA  then
                    Estimates.setfillcolor(!row,18) yellow
               endif
          !gamma=Exp_Est(!r,10)
          !delta=Exp_Est(!r,13)
               if !gamma=NA then
                    Estimates(!row,4)="Estimation impossible"
               else 
                         for !k=1 to 17
                              Estimates(!row,3+!k)=Exp_Est(!r,!k)
                         next
                    Estimates(!row,21)=NA
                    Estimates(!row,22)=NA
                         if !gamma<700 then
                              Estimates(!row,21)=(exp(!gamma)-1)*100   'Disparty, %
                                   if !delta<0 then
                                        !halflife=log(log(0.5*(exp(!gamma)+1))/!gamma)/!delta
                                   endif
                                   if !delta>0 then                                  
                                             if exp(!gamma)<=0.5 then
                                                  !gamma=-!gamma
                                             endif
                                        !halflife=log(log(2*exp(!gamma)-1)/!gamma)/!delta
                                        Estimates.setfillcolor(!row,16) red
                                        Estimates.setfillcolor(!row,22) red
                                   endif '!delta>0
                              Estimates(!row,22)=!halflife
                         endif '!gamma<700
               endif '!gamma=NA

'----------------------- fractional trend
          !row=(!r-1)*5+4          
          Estimates((!r-1)*5+4,3)="fract"
               if !SSR_Fract<1e10 and !SSR_Fract<=!SSR_Ln and !SSR_Fract<=!SSR_Exp then
                    Estimates.setfont(!row,3) +b            
               endif
               if Fract_Est(!r,5)>!Significance_level OR Fract_Est(!r,5)=NA then
                    Estimates.setfillcolor(!row,8) yellow
               endif
               if Fract_Est(!r,8)>!Significance_level  OR Fract_Est(!r,8)=NA then
                    Estimates.setfillcolor(!row,11) yellow
               endif
               if Fract_Est(!r,12)>!Significance_level  OR Fract_Est(!r,12)=NA then
                    Estimates.setfillcolor(!row,15) yellow
               endif
               if Fract_Est(!r,15)>!Significance_level  OR Fract_Est(!r,15)=NA then
                    Estimates.setfillcolor(!row,18) yellow
               endif
          !gamma=Fract_Est(!r,10)
          !delta=Fract_Est(!r,13)
               if !gamma=NA then
                    Estimates(!row,4)="Estimation impossible"
               else 
                         for !k=1 to 17
                              Estimates(!row,3+!k)=Fract_Est(!r,!k)
                         next 
                    Estimates(!row,21)=NA
                    Estimates(!row,22)=NA
                          if !gamma<700 then
                              Estimates(!row,21)=(exp(!gamma)-1)*100   'Disparty, %
                                   if !delta>0 then
                                        !halflife=(!gamma/log(0.5*(exp(!gamma)+1))-1)/!delta
                                   endif
                                   if !delta<0 then                                  
                                             if exp(!gamma)<=0.5 then
                                                  !gamma=-!gamma
                                             endif
                                        !halflife=(!gamma/log(2*exp(!gamma)-1)-1)/!delta
                                        Estimates.setfillcolor(!row,16) red
                                        Estimates.setfillcolor(!row,22) red
                                   endif
                              Estimates(!row,22)=!halflife
                         endif  '!gamma<700
               endif '!gamma=NA

'----------------------- AR(1) with constant
          Series Yrs={%0}(!r)-{%1}
          !row=(!r-1)*5+5
          Estimates(!row,3)="ARc"
          Equation ARc.ls D(Yrs)=c(1)*Yrs(-1)+c(2)
          Estimates(!row,4)=c(1)
          Estimates(!row,5)=@stderrs(1)
          Estimates(!row,6)=@tstats(1)
          Estimates(!row,13)=c(2)
          Estimates(!row,14)=@stderrs(2)
          !p=@tdist(@tstats(2), @regobs-@ncoef)
          Estimates(!row,15)=!p 
               if !p>!Significance_level then
                    Estimates.setfillcolor(!row,15) yellow
               endif
          Estimates(!row,21)=(exp(-c(2)/c(1))-1)*100 'Disparity
          Estimates(!row,19)=@rbar2
          Estimates(!row,20)=@ssr 

          '----  hac=Spectral OLS AR !		
          URoot(PP, const, hac={%AR}, info=MSIC, save=PP) Yrs 'PP test
          Estimates(!row,7)=PP(3,1)  'adjasted t-stat
          Estimates(!row,8)=PP(4,1)  'p-value
               if PP(4,1)>!Significance_level then
                    Estimates.setfillcolor(!row,8) yellow
               endif
          Estimates(!row,9)=PP(2,1)  'bandwidth

          URoot(ADF, const, info=MSIC, save=ADF) Yrs 'ADF test
          Estimates(!row,10)=ADF(3,1)  'adjasted t-stat
          Estimates(!row,11)=ADF(4,1)  'p-value
               if ADF(4,1)>!Significance_level then
                    Estimates.setfillcolor(!row,11) yellow
               endif
          Estimates(!row,12)=ADF(2,1)  'lags
               if PP(4,1)<=!Significance_level and ADF(4,1)<=!Significance_level and !p<!Significance_level then
                    Estimates.setfont(!row,3) +b
               endif

'----------------------- AR(1) with no constant
          !row=(!r-1)*5+6
          Estimates(!row,3)="ARo"
          Equation ARo.ls D(Yrs)=c(1)*Yrs(-1)
          Estimates(!row,4)=c(1)
          Estimates(!row,5)=@stderrs(1)
          Estimates(!row,6)=@tstats(1)
          Estimates(!row,19)=@rbar2 
          Estimates(!row,20)=@ssr 

          '----  hac=Spectral OLS AR !	
          URoot(PP, none, hac={%AR}, info=MSIC, save=PP) Yrs
          Estimates(!row,7)=PP(3,1)  'adjusted t-stat
          Estimates(!row,8)=PP(4,1)  'p-value
               if PP(4,1)>!Significance_level then
                    Estimates.setfillcolor(!row,8) yellow
               endif
          Estimates(!row,9)=PP(2,1)  'bandwidth

          URoot(ADF, none, info=MSIC, save=ADF) Yrs
          Estimates(!row,10)=ADF(3,1)  'adjusted t-stat
          Estimates(!row,11)=ADF(4,1)  'p-value
               if ADF(4,1)>!Significance_level then
                    Estimates.setfillcolor(!row,11) yellow
               endif
          Estimates(!row,12)=ADF(2,1)  'lags
               if PP(4,1)<=!Significance_level and ADF(4,1)<=!Significance_level then
                    Estimates.setfont(!row,3) +b
               endif
     endif '!r<>!Benchmark
     next '!r

'************************* FINISHING
delete coefNL
delete Yrs Y_detrended
delete eps
delete ARo ARc Art ARt1 ARt2 ARo_eps
delete Detrend*
delete  Log_exp_trend  Exp_trend  Fract_trend
delete NL_ 
delete p
delete tau_log  tau_exp  tau_fract
delete PP ADF
delete Ln_Est  Exp_Est  Fract_Est

Estimates(!N_*5+2,1)="Benchmark location: "+%1
Estimates(!N_*5+4,1)=Tau_Status(1,1)
Estimates(!N_*5+5,1)=Tau_Status(2,1)
Estimates(!N_*5+6,1)=Tau_Status(3,1)
Delete Tau_Status
!Elapsed=@round(@toc/60)
%Time=%Time+@str(!Elapsed)+" min." 
Estimates(!N_*5+3,1)=%Time
rename Estimates Estimates_{%1}

StatusLine %Time  
'END of program
